<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\EventBundle\Entity;

use Chill\EventBundle\Repository\EventThemeRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;

/**
 * Class EventTheme.
 */
#[ORM\HasLifecycleCallbacks]
#[ORM\Entity(repositoryClass: EventThemeRepository::class)]
#[ORM\Table(name: 'chill_event_event_theme')]
class EventTheme
{
    #[ORM\Column(type: Types::BOOLEAN, nullable: false)]
    private bool $isActive = true;

    #[ORM\Id]
    #[ORM\Column(name: 'id', type: Types::INTEGER)]
    #[ORM\GeneratedValue(strategy: 'AUTO')]
    private ?int $id = null;

    #[ORM\Column(type: Types::JSON)]
    private array $name;

    /**
     * @var Collection<int, EventTheme>
     */
    #[ORM\OneToMany(mappedBy: 'parent', targetEntity: EventTheme::class)]
    private Collection $children;

    #[ORM\ManyToOne(targetEntity: EventTheme::class, inversedBy: 'children')]
    private ?EventTheme $parent = null;

    #[ORM\Column(name: 'ordering', type: Types::FLOAT, options: ['default' => '0.0'])]
    private float $ordering = 0.0;

    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->children = new ArrayCollection();
    }

    /**
     * Get active.
     */
    public function getIsActive(): bool
    {
        return $this->isActive;
    }

    /**
     * Get id.
     */
    public function getId(): ?int
    {
        return $this->id;
    }

    /**
     * Get label.
     */
    public function getName(): array
    {
        return $this->name;
    }

    public function setIsActive(bool $active): self
    {
        $this->isActive = $active;

        return $this;
    }

    public function setName(array $label): self
    {
        $this->name = $label;

        return $this;
    }

    public function addChild(self $child): self
    {
        if (!$this->children->contains($child)) {
            $this->children[] = $child;
        }

        return $this;
    }

    public function removeChild(self $child): self
    {
        if ($this->children->removeElement($child)) {
            // set the owning side to null (unless already changed)
            if ($child->getParent() === $this) {
                $child->setParent(null);
            }
        }

        return $this;
    }

    public function getChildren(): Collection
    {
        return $this->children;
    }

    public function hasChildren(): bool
    {
        return 0 < $this->getChildren()->count();
    }

    public function hasParent(): bool
    {
        return null !== $this->parent;
    }

    public function getOrdering(): float
    {
        return $this->ordering;
    }

    public function setOrdering(float $ordering): EventTheme
    {
        $this->ordering = $ordering;

        return $this;
    }

    public function getParent(): ?self
    {
        return $this->parent;
    }

    public function setParent(?self $parent): self
    {
        $this->parent = $parent;

        $parent?->addChild($this);

        return $this;
    }
}
